local addon, w = ...




local function HideNext(id)
	local frame = _G["ESDropDown"..(id+1)]
	if ( frame ) then 
		frame:Hide() 
	end

end

local function ExpandMe(self)
	local parent = self:GetParent()
	local id = parent.id
	if ( type(self.value) ~= "table" ) then
		HideNext(id)
		if ( self.fs:GetText() ~= CANCEL and parent.t.tooltip ) then parent.t.tooltip(self)
		end
	else
		if ( parent.timer ) then 
			TimerStart(parent, parent.timer) 
		end
--[[
		local d = DropDownMe({
			t = self.value,
			f = parent.t.f,
			point = "EXPAND",
			id = id+1,
			owner = self,
		}, unpack(parent.datas))
]]
		ESUpdateDropDown(self.value, parent.t.f, "EXPAND", id+1, parent.i, parent.j, self)
	end
end

local function ClickMe(button)
	local parent = button:GetParent()
	if ( button.fs:GetText() ~= CANCEL ) then
		local value = button.value
		if ( type(value) ~= "table" ) then
			(parent.t.f or NoFunction)(button.key, value, parent.i, parent.j, parent, button)
		else
			ExpandMe(button)
		end
	else
		parent:Hide()
	end
end

local function NewDropDownButton(frame, i)
	local button = CreateFrame("Button", nil, frame)
	button:SetHeight(12)

	button:SetPoint("TOP", 0, -(i-1)*12-5)

	button:SetPoint("LEFT", frame, "LEFT", 3, 0)
	button:SetPoint("RIGHT", _G[frame:GetName().."ScrollBar"], "LEFT", -3, 0)


	table.insert(frame.buttons, button)
	
	button:SetScript("OnClick", ClickMe)
	button:SetScript("OnEnter", ExpandMe)
	button:SetScript("OnLeave", HideGameToolTip)


	button:SetHighlightTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")

	local fs = button:CreateFontString()

	--fs:SetFontObject("GameFontNormal")
	fs:SetFont("Interface\\AddOns\\"..addon.."\\lucon.ttf", 10)
	fs:SetPoint("LEFT")
	fs:SetJustifyH("LEFT")
	button.fs = fs



	local expand = button:CreateTexture(nil, "OVERLAY")

	expand:SetWidth(12)
	expand:SetHeight(12)

	expand:SetTexture("Interface\\ChatFrame\\ChatFrameExpandArrow.blp")
	expand:SetPoint("RIGHT")



	button.expand = expand

	return button 
end



local function UpdateDD(buttons, t, k, v)

	for i = 1, 20 do
		local button = buttons[i]

		if ( k ) then
			local text
			if ( type(v) == "table" ) then
				text = k
				button.expand:Show()
			else
				text = v
				button.expand:Hide()
			end
			button.key = k
			button.value = v
			button.fs:SetText(text)
			button:Show()
			k, v = next(t, k)	
		else
			button.fs:SetText(CANCEL)
			button.expand:Hide()
			button.key = nil
			button.value = nil
			button:Show()
			for j = i+1, 20 do
				buttons[j]:Hide()	
			end
			return
		end
	
	end
end

local function CountMe(t, value)
	local i = 1
	for k, v in next, t do
		if ( i == value ) then
			return k, v
		end
		i = i+1
	end
end

local function ScrollMe(self, value)
	local parent = self:GetParent()
	local t = parent.t
	UpdateDD(parent.buttons, t.t, CountMe(t.t, value))
	HideNext(parent.id)
end

local function PushMe(self)
	local parent = self:GetParent()
	local i = parent:GetValue()+self.i
	local min, max = parent:GetMinMaxValues()
	if ( i >= min and i <= max  ) then
		parent:SetValue(i)		
	end
end

local function NewDropDownFrame(id)
	local frame = CreateFrame("Frame", "ESDropDown"..id, nil)
	frame:SetScript("OnHide", function(self) 
		self:Hide()
		self:SetParent(nil)
	end)

	frame.id = id
	frame.buttons = {}


	local scroll = CreateFrame("Slider", frame:GetName().."ScrollBar", frame, "UIPanelScrollBarTemplate")
	scroll:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -3, -17)
	scroll:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -3, 17)

	scroll:SetScript("OnValueChanged", ScrollMe)
	scroll:SetValueStep(1)

	local b = _G[scroll:GetName().."ScrollUpButton"]
	b:SetScript("OnClick", PushMe)
	b.i = -1

	local b = _G[scroll:GetName().."ScrollDownButton"]
	b:SetScript("OnClick", PushMe)
	b.i = 1




	frame:SetBackdrop({edgeFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeSize = 1})

	frame:SetFrameLevel(10)

	for i = 1, 20 do
		NewDropDownButton(frame, i)	
	end
	local texture = frame:CreateTexture(nil, "BACKGROUND")
	texture:SetAllPoints(frame)
	texture:SetTexture(0.1, 0.5, 0.7, 0.75)
	return frame
end


local function UpdateWidth(expand, text, w)
	local l = #text*6.5+expand
	if ( not w or l > w ) then
		return l
	else
		return w
	end
end


function DropDownMe(t, ...)
	local v 

	v = t.id or 1
	local frame = _G["ESDropDown"..v] or NewDropDownFrame(v)

	v = t.owner
	if ( t.toggle and frame:GetParent() == v ) then
		frame:SetParent(nil)
		frame:Hide()
		return
	else
		frame:SetParent(v)
		frame:Show()
	end


	v = t.timer
	if ( v ) then
		TimerInit(frame, frame.Hide, v)
	end
	frame.timer = v


	frame.datas = {...}
	frame.t = t

	local w
	local i = 1

	for k, v in next, t.t do
		local text
		local expand
		if ( type(v) == "table" ) then
			text = k
			expand = 12
		else
			text = v
			expand = 0
		end
		
		w = UpdateWidth(expand, text, w)
		i = i+1
	end

	w = UpdateWidth(0, CANCEL, w)
	w = w+6+3

	local scroll = _G[frame:GetName().."ScrollBar"]
	if ( i > 20 ) then
		frame:SetHeight(12*20+10)
		scroll:SetWidth(16)
		scroll:Show()	
		scroll:SetMinMaxValues(1, i-20+1)

		if ( scroll:GetValue() == 1 ) then
			ScrollMe(scroll, 1)
		else

			scroll:SetValue(1)
		end

		w = w+16+3
	else
		frame:SetHeight(12*i+10)
		scroll:SetWidth(0)
		scroll:Hide()
		UpdateDD(frame.buttons, t.t, next(t.t))
	end
	frame:SetWidth(w)


	v = t.point
	frame:ClearAllPoints()
	    if ( not v or v == "CURSOR" ) then
		local s = UIParent:GetEffectiveScale()
		local x, y = GetCursorPosition()
		frame:SetPoint("TOP", UIParent, "BOTTOMLEFT", x/s, -10+y/s)
	elseif ( v == "EXPAND" ) then

		local button = t.owner
		local p = button:GetParent() 
		local r = p:GetRight()


		if ( r+w > GetScreenWidth() ) then
			frame:SetPoint("RIGHT", button, "LEFT", -3+1, 0)
		else
			frame:SetPoint("LEFT", button, "RIGHT", r-button:GetRight()-1, 0)

		end
	else
		frame:SetPoint(unpack(v))
	end
	return frame
end



function ESUpdateDropDown(t, f, point, id, i, j, owner, tooltip)

	local frame = DropDownMe({
		t = t,
		f = f,
		point = point,
		id = id,
		owner = owner,
		tooltip = tooltip,
	}, i, j)

	frame.i = i
	frame.j = j

	
	return frame
end
