local addon, w = ...
local bind
local handler = WonderBarHandler
local mods = {"ALT","RALT","LALT","SHIFT","LSHIFT","RSHIFT", "CTRL","RCTRL","LCTRL"}

function IsModifier(k) 
	for _, v in next, mods do
		if ( k == v ) or ( k == v:lower() ) then
			return true
		end
	end
end

function CatchKey(k)
	local prefix
	if ( IsModifier(k) ) then
		return k:lower()
	
	else
		    if ( k == "MiddleButton" ) then

			k = "BUTTON3"
		elseif ( k == "Button4" ) then
 
			k = "BUTTON4"
		elseif ( k == "Button5" ) then
 
			k = "BUTTON5"
		elseif ( k == -1 ) then
			k = "MOUSEWHEELDOWN"
		elseif ( k == 1 ) then
			k = "MOUSEWHEELUP"
		end
	 
		    if ( IsLeftAltKeyDown() ) then
			prefix = "ALT-"
		elseif ( IsRightAltKeyDown() ) then
			prefix = "ALT-"
		elseif ( IsLeftShiftKeyDown() ) then
			prefix = "SHIFT-"
		elseif ( IsRightShiftKeyDown() ) then
			prefix = "SHIFT-"
		elseif ( IsLeftControlKeyDown() ) then
			prefix = "CTRL-"
		elseif ( IsRightControlKeyDown() ) then
			prefix = "CTRL-"
		else
			prefix = ""
		end
		return prefix..k
	end
end

local function DisableKeyBinder(self)
	self:EnableKeyboard(false)
	self:EnableMouseWheel(false)
	self.state:Hide()
	if ( self:GetParent():GetName() ~= addon.."PetBar" ) then
		w.UpdateButton(self, self:GetAttribute("action"))
	else
		self:SetAlpha(1.0)
	end
	bind = nil
end


local function BindKey(self, k)
	if ( k == "ESCAPE" or InCombatLockdown() ) then
		DisableKeyBinder(self)
	elseif ( not IsModifier(k) and k ~= "LeftButton" and k ~= "RightButton" ) then
		k = CatchKey(k)
		DisableKeyBinder(self)
		local b = self:GetName()
		local o = WonderBarBinds[k]
		if ( o ~= b ) then
			if ( o ) then
				SetOverrideBinding(_G[o], true, k) 
			end
			SetOverrideBindingClick(self, true, k, b, k)
			WonderBarBinds[k] = b
		else
			SetOverrideBinding(self, true, k) 
			WonderBarBinds[k] = nil
		end
	end
end

local function EnableKeyBinder(self)
	if ( bind ~= self and not InCombatLockdown() ) then
		self:EnableKeyboard(true)
		self:EnableMouseWheel(true)
		self.state:Show()
		if ( bind ) then 
			DisableKeyBinder(bind) 
		end
		bind = self
	else
		DisableKeyBinder(self)
	end
end

local function NewButton(k)
	local button = CreateFrame("Button", addon.."VirtualButton"..k, nil, "SecureFrameTemplate")
	handler:WrapScript(button, "OnClick", "control:Run(scripts[tostring(down)], button)")  
	button:RegisterForClicks("AnyUp", "AnyDown")
	return button
end

function CompileVirtualButtons(t)
	for k, _ in next, t do
		if ( IsModifier(k) ) then
			if ( not handler:GetAttribute("_onstate-"..k ) ) then
	
			RegisterStateDriver(handler, k, "[modifier:"..k.."] true; false")
				handler:SetAttribute("_onstate-"..k, w.OnModifierChanged)
			end
		else
			local button = _G[addon.."VirtualButton"..k] or NewButton(k)
			SetOverrideBindingClick(button, false, k, button:GetName(), k)
		end	
	end
end

local function LoadBinds()
	for k, b in next, WonderBarBinds do
		local button = _G[b]
		if ( button ) then
			local s 
			if ( button:GetParent():GetName() == addon.."PetBar" ) then
				s = "LeftButton"
			else
				s = k
			end
			SetOverrideBindingClick(button, true, k, b, s)
		else
			WonderBarBinds[k] = nil
		end
		
	end
	for _, v in next, WonderBar do
		CompileVirtualButtons(v.modifiers)
		CompileVirtualButtons(v.units)
	end
end

RegisterEvent(LoadBinds, "VARIABLES_LOADED")


w.BindKey = BindKey
w.EnableKeyBinder = EnableKeyBinder
w.LoadBindsBar = LoadBindsBar





