local addon, w = ...
local t = w.t


local number = w.frame_parent:CreateFontString()
local button_save = CreateFrame("Button", nil, w.frame_parent)
local button_del = CreateFrame("Button", nil, w.frame_parent)
local button_reset = CreateFrame("Button", nil, w.frame_parent)

w.number = number

local function GetNumStates()
	local i = 0
	for k, v in next, w.states do
		i = i+1
	end
	return i
end
number:SetFontObject("ChatFontNormal")
number:SetPoint("TOPRIGHT", -5, -25)
number:SetJustifyH("RIGHT")

function number:Update()
	local geo = w.datas.geo
	self:SetText("("..tostring(GetNumStates()*geo.lines*geo.columns):color("88ff88").." actions)")
end


local default = {
	units = {},
	actions = {}, 
	modifiers = {}, 
	geo = {
		lines = 1,
		columns = 10,
		space = 2,	
		size = 36,
	}, 
	stances = {
		[UnitRace("player")] = 1
	}
}


local function ResetMe()

	if ( InCombatLockdown() ) then w.stop() return end
	if ( not w.saved ) then		
		local b = w.current
		local t = WonderBar[b]


		wipe(w.states)

		w:DelActions()
		w:DelModifiers()
		w:DelUnits()
		w:DelButtons(w.current, t.geo.lines*t.geo.columns+1)



 		CompileGeo(b, t)
		CompileActions(b, t)
		CompileStances(b, t)
		CompileModifiers(b, t)
		CompileUnits(b, t)

		print("Your bar "..w.current:color("88ff88").." is reverted.")
		
		w.saved = true	

	end
end


local function SecuretMe(_, f, t)
	ResetMe()
	f(unpack(t))
end



local function SecureLoad(f, ...)
	if ( w.current and not w.saved ) then
		OpenMe("PROMPT", "Your bar "..w.current:color("88ff88").." is not saved, do you want to continue ? Your bar will be reverted.", SecuretMe, f, {...}) 		
	else
		f(...)
	end
end

function w:CopyConfigBar(bar, t)
	if ( w.current ) then
		_G[w.current].bg:SetTexture(unpack(BgColor))
	end


	_G[bar].bg:SetTexture(0.6,0.4,0.4,0.5)

	if ( bar == w.parent_addon.."PetBar" ) then
		table.copy(CONFIG_PETBAR.geo, wipe(self.datas.geo))
	else
		for k, v in next, t do
			table.copy(v, wipe(self.datas[k]))
		end

		wipe(self.states)
	
		for _, t in next, t.modifiers do
			for _, v in next, t do
				self:AddState(v)
			end
			end
		for k, v in next, t.stances do
			self:AddState(v)
		end

	end
	w.saved = true
	w.current = bar
	self.frame_parent:UpdateFrame()
	self.TabMe()
end




local function SaveMe()
	if ( not w.saved ) then
		WonderBar[w.current] = table.copy(w.datas, {})
		print("Your bar "..w.current:color("88ff88").." is saved.")
		w.saved = true
	end
	
end



local function CreateMe(value)
	
	if ( InCombatLockdown() ) then w.stop() return end
	WonderBarNewBar(value)
	w:CopyConfigBar(value, default)

	w:AddActions(value, w.datas)

	CompileGeo(value, w.datas)
	CompileActions(value, w.datas)
	CompileStances(value, w.datas)
	SaveMe()
	w.tabs[1]:Click()
end


local function NewBar(self)
	local value = _G[self:GetName().."EditBox"]:GetText()
	if ( value and value ~= "" and not _G[value] ) then
		SecureLoad(CreateMe, value)
	else
		print(string.color("Wrong name.", "ff0000"))
	end
end




local box = w.NewBox(w.frame_parent)


function w.frame_parent:UpdateFrame()

	if ( not w.current or w.current == w.parent_addon.."PetBar" ) then
		button_save:Hide()
		button_reset:Hide()
		button_del:Hide()
		number:Hide()
	else
		button_save:Show()
		button_reset:Show()
		button_del:Show()
		number:Show()
	end
	if ( w.current ) then
		self.box:SetText("Your selected bar is "..string.link(w.current, "select", nil, "00ffff"))
	else
		self.box:SetText("Create a "..string.link("new bar", "newbar", "New bar", "00ffff").." or "..string.link("select one", "select", nil, "00ffff"))
	end

end

local function SelectMe(self, value, _, _, parent)
	if ( value == "New bar" ) then
		OpenMe("INPUT", "Enter a name for your bar", NewBar)
	else
		SecureLoad(w.CopyConfigBar, w, value, WonderBar[value])
	end
	if ( parent ) then
		parent:Hide()
	end
end

box.newbar = SelectMe
 
button_reset:SetScript("OnClick", function() 
	SelectMe(_, w.current)
end)


box.select = function(self, value)
	wipe(t)
	for k, v in next, WonderBar do
		table.insert(t, k)
	end
	table.insert(t, w.parent_addon.."PetBar")
	table.insert(t, "New bar")
	w.ESUpdateDropDown(t, SelectMe, "CURSOR", 1)
end


w.frame_parent:SetScript("OnHide", function()
	if ( w.current ) then
		if ( not w.saved ) then
			print("Your bar "..w.current:color("88ff88").." is not not saved. You can save it "..string.link('here', addon, 'save', 'ff00ff').." or open the "..string.link("menu", addon, "menu",'ff00ff')..'.')
		else

			_G[w.current].bg:SetTexture(unpack(BgColor))
		end
		
	end
end)



local f = ChatFrame_OnHyperlinkShow
function ChatFrame_OnHyperlinkShow (...)
	local type, value = select(2, ...):pop(":")
	if ( type == addon ) then
		if ( value == 'save' ) then
			SaveMe()
			_G[w.current].bg:SetTexture(unpack(BgColor))
		else
			w.frame_parent:Show()
		end
	else
		f(...)
	end
end



local button = CreateFrame("Button", nil, w.frame_parent)
button:SetPoint("TOPRIGHT")
button:SetWidth(24)
button:SetHeight(24)



button:SetNormalTexture("Interface\\FriendsFrame\\UI-Toast-CloseButton-Up")
button:SetPushedTexture("Interface\\FriendsFrame\\UI-Toast-CloseButton-Down")
button:SetHighlightTexture("Interface\\FriendsFrame\\UI-Toast-CloseButton-Highlight")
button:SetScript("OnClick", function(self) self:GetParent():Hide() end)


button_save:SetWidth(48)
button_save:SetHeight(48)
button_save:SetPoint("BOTTOMLEFT", 5, 5)
button_save:SetScript("OnClick", SaveMe)

local tex = button_save:CreateTexture()
tex:SetPoint("CENTER")
tex:SetWidth(48)
tex:SetHeight(48)
tex:SetTexture("Interface\\AddOns\\"..addon.."\\button_ok.tga")
tex:SetBlendMode("ADD")
button_save:SetNormalTexture(tex)

button_save.tooltip = "save"
button_save:SetScript("OnEnter", w.ShowGameTooltip)
button_save:SetScript("OnLeave", HideGameTooltip)



local function DeleteMe()
	
	if ( InCombatLockdown() ) then w.stop() return end
	if ( not w.current ) then
		return
	end
	wipe(w.states)
	w:DelActions(w.current, w.datas)
	w:DelModifiers()
	w:DelUnits()
	w:DelButtons(w.current, 1)

	
	UnregisterStateDriver(WonderBarHandler, w.current:lower())
	_G[w.current]:Hide()
	WonderBar[w.current] = nil

	w.current = nil

	w.frame_parent:UpdateFrame()
	w.TabMe()
end

button_del:SetWidth(48)
button_del:SetHeight(48)
button_del:SetPoint("BOTTOMRIGHT", -5, 5)
button_del:SetScript("OnClick", DeleteMe )

local tex = button_del:CreateTexture()
tex:SetPoint("CENTER")
tex:SetWidth(48)
tex:SetHeight(48)
tex:SetTexture("Interface\\AddOns\\"..addon.."\\button_cancel.tga")
tex:SetBlendMode("ADD")
button_del:SetNormalTexture(tex)





button_reset:SetWidth(48)
button_reset:SetHeight(48)
button_reset:SetPoint("BOTTOM", 0, 5)


button_reset.tooltip = "revert"
button_reset:SetScript("OnEnter", w.ShowGameTooltip)
button_reset:SetScript("OnLeave", HideGameTooltip)






local tex = button_reset:CreateTexture()
tex:SetPoint("CENTER")
tex:SetWidth(48)
tex:SetHeight(48)
tex:SetTexture("Interface\\AddOns\\"..addon.."\\button_reset.tga")
tex:SetBlendMode("ADD")
button_reset:SetNormalTexture(tex)

button_del.tooltip = "delete"
button_del:SetScript("OnEnter", w.ShowGameTooltip)
button_del:SetScript("OnLeave", HideGameTooltip)












w.frame_parent:UpdateFrame()
